#pragma once
/*
 *  Copyright (C) 2024  Brett Terpstra
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "blt/std/types.h"

#ifndef BLT_WITH_GRAPHICS_FONT_H
#define BLT_WITH_GRAPHICS_FONT_H

#include <ft2build.h>
#include FT_FREETYPE_H
#include <string>
#include <string_view>
#include <memory>
#include <optional>

namespace blt::gfx::font
{
    extern const unsigned int default_font_compressed_data[];
    extern const unsigned int default_font_compressed_size;
    
    FT_Library* init();
    
    inline void destroy(FT_Library* lib)
    {
        FT_Done_FreeType(*lib);
    }
    
    class font_face_t
    {
        public:
            explicit font_face_t(FT_Library* lib, std::string_view path);
            
            explicit font_face_t(FT_Library* lib, blt::u8* data, blt::size_t size, bool compressed = false);
            
            font_face_t& set_pixel_sizes(blt::u32 width, blt::u32 height);
            
            std::optional<int> load_char(char c);
            
            [[nodiscard]] FT_Face* get() const
            {
                return face.get();
            }
        
        private:
            // decompressed data ownership
            std::shared_ptr<blt::u8> internal_uncompressed_data;
            std::shared_ptr<FT_Face> face;
    };
    
    struct font_file_t
    {
        std::string_view path;
        blt::u32 character_min_index;
        blt::u32 character_max_index;
    };
    
}

#endif //BLT_WITH_GRAPHICS_FONT_H
